/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   I2C Driver                                                       */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   17b                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Port Control                                             */
/*==============================================================================*/
#include "Common.h"
// #include <itron.h>
// #include <kernel.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/* #define	DT_INLINE	static */

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO 1

/*==============================================================================*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*==============================================================================*/
static void _TP_BusPortInit(void)
{
	/* ex)	
			PA.DIR = 0x3f;
			PD.DIR = 0x3f;
			PA.word = 0x3f;
			PD.word = 0x3f;	
	*/
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );		/*	CLK	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );		/*	DAT0	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
}

/*==============================================================================*/
/*  Func:   portSetCLK                                                          */
/*  Desc:   Please describe the code to set the value to CLK Port.              */
/*==============================================================================*/
DT_INLINE void portSetCLK( DT_UINT dat)
{
	/* ex)
		PA.bit.b0 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   portSetDATA                                                         */
/*  Desc:   Please describe the code to set the value to DATA Port              */
/*==============================================================================*/
DT_INLINE void portSetDATA(DT_UINT dat)
{
	/* ex)
	 PA.bit.b2 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	/* ex)
		DI();
	*/ 
	taskENTER_CRITICAL();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	/* ex)
		EI();
	*/ 
	taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Func:   addEventInfo                                                        */
/*  Desc:   please describe the code to get target event                        */
/*==============================================================================*/
#if DT_ADD_EVENT_INFO
DT_INLINE DT_UINT addEventInfo(void)
{
	static int event_id = 0;

	/* describe code to get target eventinformation */
	/* ex. uITron4.0 API	*/
	//ID task_id;
	//get_tid( &p_task_id );
	//event_id = (DT_UINT)task_id;
	return ++event_id;

}
#endif

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_TARGET_TIME_BIT		0x04
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02


/*==============================================================================*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*==============================================================================*/
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStartCondition                                            */
/*  Desc:   Output Start Condition                                              */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStartCondition(void)
{
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStopCondition                                             */
/*  Desc:   Output Stop Condition                                               */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStopCondition(void)
{
	portSetDATA(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutRestartCondition                                          */
/*  Desc:   Output Restart Condition                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutRestartCondition(void)
{
	portSetCLK(0);
	portSetDATA(1);
	portSetCLK(1);
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_Bus1BitOutDrv                                                   */
/*  Desc:   1bit Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_Bus1BitOutDrv( DT_UINT dat )
{
	portSetCLK(0);
	portSetDATA( dat );
	portSetCLK(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutByteDrv                                                   */
/*  Desc:   Byte Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	_TP_Bus1BitOutDrv(dat & 0x80);
	_TP_Bus1BitOutDrv(dat & 0x40);
	_TP_Bus1BitOutDrv(dat & 0x20);
	_TP_Bus1BitOutDrv(dat & 0x10);
	_TP_Bus1BitOutDrv(dat & 0x08);
	_TP_Bus1BitOutDrv(dat & 0x04);
	_TP_Bus1BitOutDrv(dat & 0x02);
	_TP_Bus1BitOutDrv(dat & 0x01);
	_TP_Bus1BitOutDrv(0);							/* ACK */
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutByteDrv( dat >> 8 );
	_TP_BusOutByteDrv( dat );
	_TP_BusOutByteDrv( addr >> 8 );
	_TP_BusOutByteDrv( addr );
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutDrv                                                    */
/*  Desc:   Value Output Function                                               */
/*==============================================================================*/
DT_INLINE void _TP_MemoryOutDrv( unsigned char *p, DT_UINT size )
{
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}
}

/*==============================================================================*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{

#if DT_ADD_EVENT_INFO
	unsigned int event_id = 0;
#endif

	enterCritical();
	portInit();
#if DT_ADD_EVENT_INFO
	addr = addr | DT_EVTTRG_BIT;
#endif
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr, dat );
	_TP_BusOutRestartCondition();
#if DT_ADD_EVENT_INFO
	event_id = addEventInfo();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
#endif
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable Test Point                                       */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{

#if DT_ADD_EVENT_INFO
	int event_id = 0;
#endif

	portInit();
	enterCritical();
#if DT_ADD_EVENT_INFO
	addr = addr | DT_EVTTRG_BIT;
#endif
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	_TP_BusOutRestartCondition();
#if DT_ADD_EVENT_INFO
	event_id = addEventInfo();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
#endif
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	_TP_BusOutStopCondition();
	exitCritical();
}


